@@ -161,6 +161,12 @@ static int reference_start     = 0;
 static int reference_end       = 0;
 static int reference_size      = 0;
 
+/* Flags set during GC if a thread needs to be woken up.  The
+   notification is done after the world is resumed, to remove
+   the use of "unsafe" calls while threads are suspended. */
+static int notify_reference_thread;
+static int notify_finaliser_thread;
+
 /* Internal locks protecting the GC lists and heap */
 static VMLock heap_lock;
 static VMLock has_fnlzr_lock;
@@ -189,6 +195,27 @@ extern int finalize_mtbl_idx;
 extern int enqueue_mtbl_idx;
 extern int ldr_vmdata_offset;
 
+/* During GC all threads are suspended.  To safeguard against a
+   suspended thread holding the malloc-lock, free cannot be called
+   within the GC to free native memory associated with "special
+   objects".  Instead, frees are chained together into a pending
+   list, and are freed once all threads are resumed.  Note, we
+   cannot wrap malloc/realloc/free within the VM as this does
+   not guard against "invisible" malloc/realloc/free within libc
+   calls and JNI methods */
+static void **pending_free_list = NULL;
+void freePendingFrees();
+
+/* Similarly, allocation for internal lists within GC cannot use
+   memory from the system heap.  The following functions provide
+   the same API, but allocate memory via mmap */
+void *gcMemRealloc(void *addr, int new_size);
+void *gcMemMalloc(int size);
+void gcMemFree(void *addr);
+
+/* Cached system page size (used in above functions) */
+static int sys_page_size;
+
 /* The possible ways in which a reference may be marked in
    the mark bit array */
 #define HARD_MARK               3
@@ -245,7 +272,8 @@ void initialiseAlloc(InitArgs *args) {
     min = max;
     if(mem == NULL) {
 #else
-    char *mem = (char*)mmap(0, args->max_heap, PROT_READ|PROT_WRITE, MAP_PRIVATE|MAP_ANON, -1, 0);
+    char *mem = (char*)mmap(0, args->max_heap, PROT_READ|PROT_WRITE,
+                                               MAP_PRIVATE|MAP_ANON, -1, 0);
     if(mem == MAP_FAILED) {
 #endif
         perror("Aborting the VM -- couldn't allocate the heap");
@@ -260,6 +288,7 @@ void initialiseAlloc(InitArgs *args) {
 
     heapmax = heapbase+((args->max_heap-(heapbase-mem))&~(OBJECT_GRAIN-1));
 
+    /* Set initial free-list to one block covering entire heap */
     freelist = (Chunk*)heapbase;
     freelist->header = heapfree = heaplimit-heapbase;
     freelist->next = NULL;
@@ -267,12 +296,17 @@ void initialiseAlloc(InitArgs *args) {
     TRACE_GC("Alloced heap size %p\n", heaplimit-heapbase);
     allocMarkBits();
 
+    /* Initialise GC locks */
     initVMLock(heap_lock);
     initVMLock(has_fnlzr_lock);
     initVMLock(registered_refs_lock);
     initVMWaitLock(run_finaliser_lock);
     initVMWaitLock(reference_lock);
 
+    /* Cache system page size -- used for internal GC lists */
+    sys_page_size = getpagesize();
+
+    /* Set verbose option from initialisation arguments */
     verbosegc = args->verbosegc;
 }
 
@@ -303,14 +337,14 @@ void markConservativeRoot(Object *object) {
 
     if((conservative_root_count % LIST_INCREMENT) == 0) {
         int new_size = conservative_root_count + LIST_INCREMENT;
-        conservative_roots = sysRealloc(conservative_roots,
-                                        new_size * sizeof(Object *));
+        conservative_roots = gcMemRealloc(conservative_roots,
+                                          new_size * sizeof(Object *));
     }
     conservative_roots[conservative_root_count++] = object;
 }
 
 void freeConservativeRoots() {
-    free(conservative_roots);
+    gcMemFree(conservative_roots);
     conservative_roots = NULL;
     conservative_root_count = 0;
 }
@@ -502,7 +536,7 @@ void markChildren(Object *ob, int mark, int mark_soft_refs) {
     if(list##_start == list##_end) {                                     \
         list##_end = list##_size;                                        \
         list##_start = list##_size += LIST_INCREMENT;                    \
-        list##_list = (Object**)sysRealloc(list##_list,                  \
+        list##_list = (Object**)gcMemRealloc(list##_list,                \
                                            list##_size*sizeof(Object*)); \
     }                                                                    \
     list##_end = list##_end%list##_size;                                 \
@@ -582,11 +616,9 @@ static void doMark(Thread *self, int mark_soft_refs) {
     if(j != has_finaliser_count) {
         has_finaliser_count = j;
 
-        /* Extra finalizers to be ran, so signal the finalizer thread
-           in case it needs waking up.  It won't run until it's
-           resumed */
-
-        notifyAllVMWaitLock(run_finaliser_lock, self);
+        /* Extra finalizers to be ran, so we need to signal the
+           finalizer thread in case it needs waking up. */
+        notify_finaliser_thread = TRUE;
     }
 
     /* Mark the objects waiting to be finalized.  We must mark them
@@ -647,7 +679,9 @@ int handleMarkedSpecial(Object *ob) {
 
             if((Object *)INST_DATA(ob)[ref_queue_offset] != NULL) {
                 TRACE_GC("FREE: Adding to list for enqueuing.\n");
+
                 ADD_TO_OBJECT_LIST(reference, ob);
+                notify_reference_thread = TRUE;
             }
         }
     }
@@ -673,7 +707,7 @@ void handleUnmarkedSpecial(Object *ob) {
                 /* Free the native thread structure (see comment
                    in detachThread (thread.c) */
                 TRACE_GC("FREE: Freeing native thread for VMThread object %p\n", ob);
-                free(threadSelf0(ob));
+                gcPendingFree(threadSelf0(ob));
             }
 }
 
@@ -885,7 +919,7 @@ void addConservativeRoots2Hash() {
     for(i = 1; i < conservative_root_count; i <<= 1);
     con_roots_hashtable_size = i << 1;
 
-    con_roots_hashtable = sysMalloc(con_roots_hashtable_size * sizeof(uintptr_t));
+    con_roots_hashtable = gcMemMalloc(con_roots_hashtable_size * sizeof(uintptr_t));
     memset(con_roots_hashtable, 0, con_roots_hashtable_size * sizeof(uintptr_t));
 
     for(i = 0; i < conservative_root_count; i++) {
@@ -1106,7 +1140,9 @@ int threadChildren(Object *ob, Object *new_addr) {
 
                         if(body[ref_queue_offset] != NULL) {
                             TRACE_GC("Adding to list for enqueuing.\n");
+
                             ADD_TO_OBJECT_LIST(reference, new_addr);
+                            notify_reference_thread = TRUE;
                         }
 out:
                         if(!cleared)
@@ -1294,7 +1330,7 @@ marked_phase2:
     chunkpp = &freelist;
 
     /* Free conservative roots hash table */
-    free(con_roots_hashtable);
+    gcMemFree(con_roots_hashtable);
     
     if(verbosegc) {
         long long size = heaplimit-heapbase;
@@ -1339,11 +1375,11 @@ void expandHeap(int min) {
     new->next = NULL;
 
     if(freelist != NULL) {
-    /* The freelist is in address order - find the last
-       free chunk and add the new area to the end.  */
+        /* The freelist is in address order - find the last
+           free chunk and add the new area to the end.  */
 
-    for(chunk = freelist; chunk->next != NULL; chunk = chunk->next);
-        chunk->next = new;
+        for(chunk = freelist; chunk->next != NULL; chunk = chunk->next);
+            chunk->next = new;
     } else
         freelist = new;
 
@@ -1353,10 +1389,11 @@ void expandHeap(int min) {
     /* The heap has increased in size - need to reallocate
        the mark bits to cover new area */
 
-    free(markBits);
+    sysFree(markBits);
     allocMarkBits();
 }
 
+
 /* ------------------------- GARBAGE COLLECT ------------------------- */
 
 static void getTime(struct timeval *tv) {
@@ -1375,6 +1412,7 @@ static long endTime(struct timeval *start) {
 }
 
 unsigned long gc0(int mark_soft_refs, int compact) {
+
     Thread *self = threadSelf();
     uintptr_t largest;
 
@@ -1383,9 +1421,9 @@ unsigned long gc0(int mark_soft_refs, int compact) {
     if(compact_override)
         compact = compact_value;
 
-    /* Stop the world */
-    disableSuspend(self);
-    suspendAllThreads(self);
+    /* Reset flags.  Will be set during GC if a thread needs
+       to be woken up */
+    notify_finaliser_thread = notify_reference_thread = FALSE;
 
     /* Grab locks associated with the suspension blocked
        regions.  This ensures all threads have suspended
@@ -1401,6 +1439,10 @@ unsigned long gc0(int mark_soft_refs, int compact) {
     /* Held by the reference handler thread */
     lockVMWaitLock(reference_lock, self);
 
+    /* Stop the world */
+    disableSuspend(self);
+    suspendAllThreads(self);
+
     if(verbosegc) {
         struct timeval start;
         float mark_time;
@@ -1421,20 +1463,27 @@ unsigned long gc0(int mark_soft_refs, int compact) {
         largest = compact ? doCompact() : doSweep(self);
     }
 
-    /* Notify the reference handler.  If there's no
-       work to be done it'll go back to sleep */
-    notifyAllVMWaitLock(reference_lock, self);
-
     /* Restart the world */
     resumeAllThreads(self);
     enableSuspend(self);
 
+    /* Notify the finaliser thread if new finalisers
+       need to be ran */
+    if(notify_finaliser_thread)
+        notifyAllVMWaitLock(run_finaliser_lock, self);
+
+    /* Notify the reference thread if new references
+       have been enqueued */
+    if(notify_reference_thread)
+        notifyAllVMWaitLock(reference_lock, self);
+
     /* Release the locks */
     unlockVMLock(has_fnlzr_lock, self);
     unlockVMWaitLock(reference_lock, self);
     unlockVMWaitLock(run_finaliser_lock, self);
 
     freeConservativeRoots();
+    freePendingFrees();
 
     return largest;
 }
@@ -1535,7 +1584,7 @@ void asyncGCThreadLoop(Thread *self) {
         do {                                                                    \
             Object *ob;                                                         \
             list##_start %= list##_size;                                        \
-	    if((ob = list##_list[list##_start]) == NULL)                        \
+            if((ob = list##_list[list##_start]) == NULL)                        \
                 continue;                                                       \
                                                                                 \
             unlockVMWaitLock(list##_lock, self);                                \
@@ -1588,7 +1637,7 @@ void initialiseGC(InitArgs *args) {
 
     MethodBlock *init;
     Class *oom_clazz = findSystemClass("java/lang/OutOfMemoryError");
-    if(exceptionOccured()) {
+    if(exceptionOccurred()) {
         printException();
         exitVM(1);
     }
@@ -1614,6 +1663,7 @@ void initialiseGC(InitArgs *args) {
     compact_value = args->do_compact;
 }
 
+
 /* ------------------------- ALLOCATION ROUTINES  ------------------------- */
 
 void *gcMalloc(int len) {
@@ -2009,9 +2059,12 @@ Object *allocMultiArray(Class *array_class, int dim, intptr_t *count) {
 
 Class *allocClass() {
     Class *class = (Class*)gcMalloc(sizeof(ClassBlock)+sizeof(Class));
-    SET_SPECIAL_OB(class);
 
-    TRACE_ALLOC("<ALLOC: allocated class object @%p>\n", class);
+    if(class != NULL) {
+        SET_SPECIAL_OB(class);
+        TRACE_ALLOC("<ALLOC: allocated class object @%p>\n", class);
+    }
+
     return class; 
 }
 
@@ -2084,12 +2137,84 @@ unsigned long maxHeapMem() {
     return heapmax-heapbase;
 }
 
+
+/* ------ Allocation routines for internal GC lists ------- */
+
+/* These use mmap to avoid deadlock with threads
+    suspended while holding the malloc lock */
+
+void *gcMemMalloc(int n) {
+    int size = n + sizeof(int);
+    int *mem = mmap(0, size, PROT_READ|PROT_WRITE,
+                             MAP_PRIVATE|MAP_ANON, -1, 0);
+
+    if(mem == MAP_FAILED) {
+        jam_fprintf(stderr, "Mmap failed - aborting VM...\n");
+        exitVM(1);
+    }
+
+    *mem++ = size;
+    return mem;
+}
+
+void *gcMemRealloc(void *addr, int size) {
+    if(addr == NULL)
+        return gcMemMalloc(size);
+    else {
+        int *mem = addr;
+        int old_size = *--mem;
+        int new_size = size + sizeof(int);
+
+        if(old_size/sys_page_size == new_size/sys_page_size) {
+            *mem = new_size;
+            return addr;
+        } else {
+            int copy_size = new_size > old_size ? old_size : new_size;
+            void *new_mem = gcMemMalloc(size);
+
+            memcpy(new_mem, addr, copy_size - sizeof(int));
+            munmap(mem, old_size);
+
+            return new_mem;
+        }
+    }
+}
+
+void gcMemFree(void *addr) {
+    if(addr != NULL) {
+        int *mem = addr;
+        int size = *--mem;
+        munmap(mem, size);
+    }
+}
+
+/* Delayed free, for use while in GC to avoid deadlock with
+   threads suspended while holding the malloc lock.  This
+   simply chains the pointers into a linked-list */
+
+void gcPendingFree(void *addr) {
+    if(addr != NULL) {
+        *(void**)addr = pending_free_list;
+        pending_free_list = addr;
+    }
+}
+
+void freePendingFrees() {
+    while(pending_free_list) {
+        void *next = *pending_free_list;
+        sysFree(pending_free_list);
+        pending_free_list = next;
+    }
+}
+
+
 /* ------ Allocation from system heap ------- */
 
-void *sysMalloc(int n) {
+void *sysMalloc(int size) {
+    int n = size < sizeof(void*) ? sizeof(void*) : size;
     void *mem = malloc(n);
 
-    if(mem == NULL && n > 0) {
+    if(mem == NULL) {
         jam_fprintf(stderr, "Malloc failed - aborting VM...\n");
         exitVM(1);
     }
@@ -2097,8 +2222,8 @@ void *sysMalloc(int n) {
     return mem;
 }
 
-void *sysRealloc(void *ptr, int n) {
-    void *mem = realloc(ptr, n);
+void *sysRealloc(void *addr, int size) {
+    void *mem = realloc(addr, size);
 
     if(mem == NULL) {
         jam_fprintf(stderr, "Realloc failed - aborting VM...\n");
@@ -2108,3 +2233,7 @@ void *sysRealloc(void *ptr, int n) {
     return mem;
 }
 
+void sysFree(void *addr) {
+    free(addr);
+}
+
