@@ -33,12 +33,12 @@
 #include "lock.h"
 #include "hash.h"
 #include "zip.h"
-#include "interp.h"
 #include "class.h"
+#include "interp.h"
 
 #define PREPARE(ptr) ptr
 #define SCAVENGE(ptr) FALSE
-#define FOUND(ptr)
+#define FOUND(ptr) ptr
 
 static int verbose;
 static char *bootpath;
@@ -233,7 +233,7 @@ Class *defineClass(char *classname, char *data, int offset, int len, Object *cla
                CP_INFO(constant_pool,i) = (uintptr_t) (utf8 = findUtf8String(buff));
 
                if(utf8 != buff)
-                   free(buff);
+                   sysFree(buff);
 
                break;
            }
@@ -283,7 +283,7 @@ Class *defineClass(char *classname, char *data, int offset, int len, Object *cla
        u2 index;
        READ_TYPE_INDEX(index, constant_pool, CONSTANT_Class, ptr, len);
        interfaces[i] = resolveClass(class, index, FALSE);
-       if(exceptionOccured())
+       if(exceptionOccurred())
            return NULL; 
     }
 
@@ -535,7 +535,7 @@ Class *defineClass(char *classname, char *data, int offset, int len, Object *cla
 
     classblock->super = super_idx ? resolveClass(class, super_idx, FALSE) : NULL;
 
-    if(exceptionOccured())
+    if(exceptionOccurred())
        return NULL;
 
     classblock->state = CLASS_LOADED;
@@ -1085,6 +1085,9 @@ Class *initClass(Class *class) {
    Object *excep;
    int i;
 
+   if(cb->state >= CLASS_INITED)
+      return class;
+
    linkClass(class);
    objectLock((Object *)class);
 
@@ -1112,7 +1115,7 @@ Class *initClass(Class *class) {
    if(!(cb->access_flags & ACC_INTERFACE) && cb->super
               && (CLASS_CB(cb->super)->state != CLASS_INITED)) {
       initClass(cb->super);
-      if(exceptionOccured()) {
+      if(exceptionOccurred()) {
           objectLock((Object *)class);
           cb->state = CLASS_BAD;
           goto notify;
@@ -1135,7 +1138,7 @@ Class *initClass(Class *class) {
    if((mb = findMethod(class, "<clinit>", "()V")) != NULL)
       executeStaticMethod(class, mb);
 
-   if((excep = exceptionOccured())) {
+   if((excep = exceptionOccurred())) {
        Class *error, *eiie;
        Object *ob;
 
@@ -1186,7 +1189,7 @@ char *findFileEntry(char *path, int *file_len) {
     if(read_len == *file_len)
         return data;
 
-    free(data);
+    sysFree(data);
     return NULL;
 }
 
@@ -1213,7 +1216,7 @@ Class *loadSystemClass(char *classname) {
     }
 
     class = defineClass(classname, data, 0, file_len, NULL);
-    free(data);
+    sysFree(data);
 
     if(verbose && class)
         jam_printf("[Loaded %s from %s]\n", classname, bootclasspath[i-1].path);
@@ -1263,7 +1266,7 @@ Class *findSystemClass0(char *classname) {
    if(class == NULL)
        class = loadSystemClass(classname);
 
-   if(!exceptionOccured())
+   if(!exceptionOccurred())
        linkClass(class);
 
    return class;
@@ -1272,7 +1275,7 @@ Class *findSystemClass0(char *classname) {
 Class *findSystemClass(char *classname) {
    Class *class = findSystemClass0(classname);
 
-   if(!exceptionOccured())
+   if(!exceptionOccurred())
        initClass(class);
 
    return class;
@@ -1339,7 +1342,7 @@ Class *findNonArrayClassFromClassLoader(char *classname, Object *loader) {
         Object *string = createString(dot_name);
         Object *excep;
 
-        free(dot_name);
+        sysFree(dot_name);
         if(string == NULL)
             return NULL;
 
@@ -1355,7 +1358,7 @@ Class *findNonArrayClassFromClassLoader(char *classname, Object *loader) {
         class = *(Class**)executeMethod(loader,
                     CLASS_CB(loader->class)->method_table[loadClass_mtbl_idx], string);
 
-        if((excep = exceptionOccured())) {
+        if((excep = exceptionOccurred())) {
             clearException();
             signalChainedException("java/lang/NoClassDefFoundError", classname, excep);
             return NULL;
@@ -1382,14 +1385,14 @@ Class *findClassFromClassLoader(char *classname, Object *loader) {
 Object *getSystemClassLoader() {
     Class *class_loader = findSystemClass("java/lang/ClassLoader");
 
-    if(!exceptionOccured()) {
+    if(!exceptionOccurred()) {
         MethodBlock *mb;
 
         if((mb = findMethod(class_loader, "getSystemClassLoader",
                                           "()Ljava/lang/ClassLoader;")) != NULL) {
             Object *system_loader = *(Object**)executeStaticMethod(class_loader, mb);
 
-            if(!exceptionOccured()) 
+            if(!exceptionOccurred()) 
                 return system_loader;
         }
     }
@@ -1454,63 +1457,71 @@ void freeClassData(Class *class) {
     int i;
 
     if(IS_ARRAY(cb)) {
-        free(cb->name);
-        free(cb->interfaces);
+        gcPendingFree(cb->name);
+        gcPendingFree(cb->interfaces);
         return;
     }
 
-    free((void*)cb->constant_pool.type);
-    free(cb->constant_pool.info);
-    free(cb->interfaces);
+    gcPendingFree((void*)cb->constant_pool.type);
+    gcPendingFree(cb->constant_pool.info);
+    gcPendingFree(cb->interfaces);
 
     for(i = 0; i < cb->fields_count; i++) {
         FieldBlock *fb = &cb->fields[i];
 
         if(fb->annotations != NULL) {
-            free(fb->annotations->data);
-            free(fb->annotations);
+            gcPendingFree(fb->annotations->data);
+            gcPendingFree(fb->annotations);
         }
     }
 
-    free(cb->fields);
+    gcPendingFree(cb->fields);
 
     for(i = 0; i < cb->methods_count; i++) {
         MethodBlock *mb = &cb->methods[i];
 
 #ifdef DIRECT
-        if(!(mb->access_flags & ACC_ABSTRACT) || !((uintptr_t)mb->code & 0x3))
+        if(!((uintptr_t)mb->code & 0x3)) {
+#ifdef INLINING
+            if(cb->state >= CLASS_LINKED)
+                freeMethodInlinedInfo(mb);
+#endif
+            gcPendingFree(mb->code);
+        } else
+            if(!(mb->access_flags & ACC_ABSTRACT))
+                gcPendingFree((void*)((uintptr_t)mb->code & ~3));
 #else
         if(!(mb->access_flags & ACC_ABSTRACT))
+            gcPendingFree(mb->code);
 #endif
-            free((void*)((uintptr_t)mb->code & ~3));
 
-        free(mb->exception_table);
-        free(mb->line_no_table);
-        free(mb->throw_table);
+        gcPendingFree(mb->exception_table);
+        gcPendingFree(mb->line_no_table);
+        gcPendingFree(mb->throw_table);
 
         if(mb->annotations != NULL) {
             if(mb->annotations->annotations != NULL) {
-                free(mb->annotations->annotations->data);
-                free(mb->annotations->annotations);
+                gcPendingFree(mb->annotations->annotations->data);
+                gcPendingFree(mb->annotations->annotations);
             }
             if(mb->annotations->parameters != NULL) {
-                free(mb->annotations->parameters->data);
-                free(mb->annotations->parameters);
+                gcPendingFree(mb->annotations->parameters->data);
+                gcPendingFree(mb->annotations->parameters);
             }
             if(mb->annotations->dft_val != NULL) {
-                free(mb->annotations->dft_val->data);
-                free(mb->annotations->dft_val);
+                gcPendingFree(mb->annotations->dft_val->data);
+                gcPendingFree(mb->annotations->dft_val);
             }
-            free(mb->annotations);
+            gcPendingFree(mb->annotations);
         }
     } 
 
-    free(cb->methods);
-    free(cb->inner_classes);
+    gcPendingFree(cb->methods);
+    gcPendingFree(cb->inner_classes);
 
     if(cb->annotations != NULL) {
-        free(cb->annotations->data);
-        free(cb->annotations);
+        gcPendingFree(cb->annotations->data);
+        gcPendingFree(cb->annotations);
     }
 
    if(cb->state >= CLASS_LINKED) {
@@ -1521,15 +1532,15 @@ void freeClassData(Class *class) {
         if(!IS_INTERFACE(cb)) {
              int spr_imthd_sze = super_cb->imethod_table_size;
 
-            free(cb->method_table);
+            gcPendingFree(cb->method_table);
             if(cb->imethod_table_size > spr_imthd_sze)
-                free(cb->imethod_table[spr_imthd_sze].offsets);
+                gcPendingFree(cb->imethod_table[spr_imthd_sze].offsets);
         }
 
-        free(cb->imethod_table);
+        gcPendingFree(cb->imethod_table);
 
         if(cb->refs_offsets_table != super_cb->refs_offsets_table)
-            free(cb->refs_offsets_table);
+            gcPendingFree(cb->refs_offsets_table);
     }
 }
 
@@ -1538,8 +1549,8 @@ void freeClassLoaderData(Object *class_loader) {
 
     if(vmdata != NULL) {
         HashTable **table = ARRAY_DATA(vmdata);
-        freeHashTable((**table));
-        free(*table);
+        gcFreeHashTable((**table));
+        gcPendingFree(*table);
     }
 }
 
@@ -1619,11 +1630,11 @@ void scanDirForJars(char *dir) {
         while(--n >= 0) {
             char *buff;
             bootpathlen += strlen(namelist[n]->d_name) + dirlen + 2;
-            buff = malloc(bootpathlen);
+            buff = sysMalloc(bootpathlen);
 
             strcat(strcat(strcat(strcat(strcpy(buff, dir), "/"), namelist[n]->d_name), ":"), bootpath);
 
-            free(bootpath);
+            sysFree(bootpath);
             bootpath = buff;
             free(namelist[n]);
         }
@@ -1649,19 +1660,9 @@ void scanDirsForJars(char *directories) {
     if(end != dirs)
         scanDirForJars(dirs);
 
-    free(dirs);
+    sysFree(dirs);
 }
 
-#ifdef USE_ZIP
-#define JAMVM_CLASSES INSTALL_DIR"/share/jamvm/classes.zip"
-#define CLASSPATH_CLASSES CLASSPATH_INSTALL_DIR"/share/classpath/glibj.zip"
-#else
-#define JAMVM_CLASSES INSTALL_DIR"/share/jamvm/classes"
-#define CLASSPATH_CLASSES CLASSPATH_INSTALL_DIR"/share/classpath"
-#endif
-
-#define DFLT_BCP JAMVM_CLASSES":"CLASSPATH_CLASSES
-
 char *setBootClassPath(char *cmdlne_bcp, char bootpathopt) {
     char *endorsed_dirs;
 
