@@ -25,6 +25,7 @@
 #include <stdarg.h>
 
 #include "jam.h"
+#include "class.h"
 
 #ifdef USE_ZIP
 #define BCP_MESSAGE "<jar/zip files and directories separated by :>"
@@ -46,6 +47,15 @@ void showNonStandardOptions() {
     printf("  -Xnoasyncgc\t   turn off asynchronous garbage collection\n");
     printf("  -Xcompactalways  always compact the heap when garbage-collecting\n");
     printf("  -Xnocompact\t   turn off heap-compaction\n");
+#ifdef INLINING
+    printf("  -Xnoinlining\t   turn off interpreter inlining\n");
+    printf("  -Xshowreloc\t   show opcode relocatability\n");
+    printf("  -Xreplication:[none|always|<value>]\n");
+    printf("\t\t   none : always re-use super-instructions\n");
+    printf("\t\t   always : never re-use super-instructions\n");
+    printf("\t\t   <value> copy when usage reaches threshold value\n");
+    printf("  -Xcodemem:[unlimited|<size>] (default maximum heapsize/4)\n");
+#endif
     printf("  -Xms<size>\t   set the initial size of the heap (default = %dM)\n", DEFAULT_MIN_HEAP/MB);
     printf("  -Xmx<size>\t   set the maximum size of the heap (default = %dM)\n", DEFAULT_MAX_HEAP/MB);
     printf("  -Xss<size>\t   set the Java stack size for each thread (default = %dK)\n", DEFAULT_STACK/KB);
@@ -84,6 +94,33 @@ void showVersionAndCopyright() {
     printf("but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
     printf("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
     printf("GNU General Public License for more details.\n");
+
+    printf("\nBuild information:\n\nExecution Engine: ");
+
+#ifdef THREADED
+#ifdef DIRECT
+#ifdef INLINING
+    printf("inline-");
+#else /* INLINING */
+    printf("direct-");
+#endif /* INLINING */
+#endif /* DIRECT */
+    printf("threaded interpreter");
+#ifdef USE_CACHE
+    printf(" with stack-caching\n");
+#else /* USE_CACHE*/
+    printf("\n");
+#endif /* USE_CACHE */
+#else /* THREADED */
+    printf("switch-based interpreter\n");
+#endif /*THREADED */
+
+#if defined(__GNUC__) && defined(__VERSION__)
+    printf("Compiled with: gcc %s\n", __VERSION__);
+#endif
+
+    printf("\nBoot Library Path: %s\n", getBootDllPath());
+    printf("Boot Class Path: %s\n", DFLT_BCP);
 }
 
 void showFullVersion() {
@@ -179,11 +216,8 @@ int parseCommandLine(int argc, char *argv[], InitArgs *args) {
         } else if(strncmp(argv[i], "-D", 2) == 0) {
             char *pntr, *key = strcpy(sysMalloc(strlen(argv[i]+2) + 1), argv[i]+2);
             for(pntr = key; *pntr && (*pntr != '='); pntr++);
-            if(pntr == key) {
-                printf("Bad property: %s\n", argv[i]);
-                goto exit;
-            }
-            *pntr++ = '\0';
+            if(*pntr)
+                *pntr++ = '\0';
             props[props_count].key = key;
             props[props_count++].value = pntr;
 
@@ -217,7 +251,34 @@ int parseCommandLine(int argc, char *argv[], InitArgs *args) {
 
         } else if(strcmp(argv[i], "-Xcompactalways") == 0) {
             args->compact_specified = args->do_compact = TRUE;
+#ifdef INLINING
+        } else if(strcmp(argv[i], "-Xnoinlining") == 0) {
+            /* Turning inlining off is equivalent to setting
+               code memory to zero */
+            args->codemem = 0;
 
+        } else if(strncmp(argv[i], "-Xreplication:", 14) == 0) {
+            char *pntr = argv[i] + 14;
+
+            if(strcmp(pntr, "none") == 0)
+                args->replication = INT_MAX;
+            else
+                if(strcmp(pntr, "always") == 0)
+                    args->replication = 0;
+                else
+                    args->replication = strtol(pntr, NULL, 0);
+
+        } else if(strncmp(argv[i], "-Xcodemem:", 10) == 0) {
+            char *pntr = argv[i] + 10;
+
+            args->codemem = strncmp(pntr, "unlimited", 10) == 0 ?
+                INT_MAX : parseMemValue(pntr);
+
+        } else if(strcmp(argv[i], "-Xshowreloc") == 0) {
+            showRelocatability();
+            status = 0;
+            goto exit;
+#endif
         } else {
             printf("Unrecognised command line option: %s\n", argv[i]);
             break;
@@ -235,12 +296,13 @@ int main(int argc, char *argv[]) {
     Object *system_loader, *array;
     MethodBlock *mb;
     InitArgs args;
+    int class_arg;
     char *cpntr;
     int status;
     int i;
 
     setDefaultInitArgs(&args);
-    int class_arg = parseCommandLine(argc, argv, &args);
+    class_arg = parseCommandLine(argc, argv, &args);
 
     args.main_stack_base = &array_class;
     initVM(&args);
@@ -260,7 +322,7 @@ int main(int argc, char *argv[]) {
     if((main_class = findClassFromClassLoader(argv[class_arg], system_loader)) != NULL)
         initClass(main_class);
 
-    if(exceptionOccured()) {
+    if(exceptionOccurred()) {
         printException();
         exitVM(1);
     }
@@ -287,9 +349,9 @@ int main(int argc, char *argv[]) {
             executeStaticMethod(main_class, mb, array);
     }
 
-    /* ExceptionOccured returns the exception or NULL, which is OK
+    /* ExceptionOccurred returns the exception or NULL, which is OK
        for normal conditionals, but not here... */
-    if((status = exceptionOccured() ? 1 : 0))
+    if((status = exceptionOccurred() ? 1 : 0))
         printException();
 
     /* Wait for all but daemon threads to die */
