@@ -24,12 +24,12 @@
 #include <limits.h>
 #include <stdio.h>
 
-/* Architecture dependent definitions */
-#include "arch.h"
-
 /* Configure options */
 #include "config.h"
 
+/* Architecture dependent definitions */
+#include "arch.h"
+
 #ifndef TRUE
 #define         TRUE    1
 #define         FALSE   0
@@ -265,6 +265,7 @@
 #define OPC_MULTIANEWARRAY_QUICK        243
 #define OPC_INVOKEINTERFACE_QUICK       244
 #define OPC_ABSTRACT_METHOD_ERROR       245
+#define OPC_INLINE_REWRITER             246
 
 #define CONSTANT_Utf8                   1
 #define CONSTANT_Integer                3
@@ -413,6 +414,30 @@ typedef struct lookup_table {
     LookupEntry *entries;
 } LookupTable;
 
+#ifdef INLINING
+typedef struct opcode_info {
+    unsigned char opcode;
+    unsigned char cache_depth;
+} OpcodeInfo;
+
+typedef struct code_block {
+    int length;
+    Instruction *start;
+    OpcodeInfo *opcodes;
+} CodeBlock;
+
+typedef struct quick_prepare_info {
+    CodeBlock block;
+    Instruction *quickened;
+    struct quick_prepare_info *next;
+} QuickPrepareInfo;
+
+typedef struct prepare_info {
+    CodeBlock block;
+    Operand operand;
+} PrepareInfo;
+#endif
+
 typedef Instruction *CodePntr;
 #else
 typedef unsigned char *CodePntr;
@@ -450,6 +475,9 @@ typedef struct methodblock {
    LineNoTableEntry *line_no_table;
    int method_table_index;
    MethodAnnotationData *annotations;
+#ifdef INLINING
+   QuickPrepareInfo *quick_prepare_info;
+#endif
 } MethodBlock;
 
 typedef struct fieldblock {
@@ -574,6 +602,10 @@ typedef struct InitArgs {
     void (*exit)(int status);
     void (*abort)(void);
 
+#ifdef INLINING
+    unsigned int codemem;
+    int replication;
+#endif
 } InitArgs;
 
 #define CLASS_CB(classRef)              ((ClassBlock*)(classRef+1))
@@ -654,7 +686,7 @@ typedef struct InitArgs {
    a StackOverflow exception */
 #define STACK_RED_ZONE_SIZE 1*KB
 
-#define JAVA_COMPAT_VERSION "1.4.2"
+#define JAVA_COMPAT_VERSION "1.5.0"
 
 /* --------------------- Function prototypes  --------------------------- */
 
@@ -681,8 +713,13 @@ extern unsigned long totalHeapMem();
 extern unsigned long maxHeapMem();
 
 extern void *sysMalloc(int n);
+extern void sysFree(void *ptr);
 extern void *sysRealloc(void *ptr, int n);
 
+extern void *gcMemMalloc(int n);
+extern void gcMemFree(void *ptr);
+extern void *gcMemRealloc(void *ptr, int n);
+
 extern void registerStaticObjectRef(Object **ob);
 extern void registerStaticObjectRefLocked(Object **ob);
 
@@ -692,6 +729,8 @@ extern void registerStaticObjectRefLocked(Object **ob);
 #define registerStaticClassRefLocked(ref) \
     registerStaticObjectRefLocked(ref);
 
+extern void gcPendingFree(void *addr);
+
 /* GC support */
 extern void threadReference(Object **ref);
 extern int isMarked(Object *ob);
@@ -770,7 +809,7 @@ extern void *executeMethodList(Object *ob, Class *class, MethodBlock *mb, u8 *ar
 
 /* excep */
 
-extern Object *exceptionOccured();
+extern Object *exceptionOccurred();
 extern void signalChainedException(char *excep_name, char *excep_mess, Object *cause);
 extern void setException(Object *excep);
 extern void clearException();
@@ -794,7 +833,7 @@ extern void initialiseException();
 /* interp */
 
 extern uintptr_t *executeJava();
-extern void initialiseInterpreter();
+extern void initialiseInterpreter(InitArgs *args);
 
 /* String */
 
@@ -802,6 +841,7 @@ extern Object *findInternedString(Object *string);
 extern Object *createString(char *utf8);
 extern Object *createStringFromUnicode(unsigned short *unicode, int len);
 extern char *String2Cstr(Object *string);
+extern char *String2Buff(Object *string, char *buff, int buff_len);
 extern int getStringLen(Object *string);
 extern unsigned short *getStringChars(Object *string);
 extern Object *getStringCharsArray(Object *string);
@@ -824,6 +864,7 @@ extern char *findUtf8String(char *string);
 extern int utf8CharLen(unsigned short *unicode, int len);
 extern char *unicode2Utf8(unsigned short *unicode, int len, char *utf8);
 extern char *slash2dots(char *utf8);
+extern char *slash2dots2buff(char *utf8, char *buff, int buff_len);
 extern void initialiseUtf8();
 
 /* Dll */
@@ -838,7 +879,7 @@ extern uintptr_t *resolveNativeWrapper(Class *class, MethodBlock *mb, uintptr_t
 extern void unloadClassLoaderDlls(Object *loader);
 extern void threadLiveClassLoaderDlls();
 
-/* Dll OS */
+/* OS */
 
 extern char *nativeLibPath();
 extern void *nativeLibOpen(char *path);
@@ -939,3 +980,8 @@ extern void jamvm_exit(int status);
 
 #define jam_printf(fmt, ...) jam_fprintf(stdout, fmt, ## __VA_ARGS__)
 
+/* inlining */
+
+extern void freeMethodInlinedInfo(MethodBlock *mb);
+extern int  initialiseInlining(InitArgs *args);
+extern void showRelocatability();
