@@ -397,7 +397,7 @@ void Jam_DeleteWeakGlobalRef(JNIEnv *env, jweak obj) {
 }
 
 jboolean Jam_ExceptionCheck(JNIEnv *env) {
-    return exceptionOccured() ? JNI_TRUE : JNI_FALSE;
+    return exceptionOccurred() ? JNI_TRUE : JNI_FALSE;
 }
 
 /* JNI 1.1 interface */
@@ -443,7 +443,7 @@ jint Jam_ThrowNew(JNIEnv *env, jclass clazz, const char *message) {
 }
 
 jthrowable Jam_ExceptionOccurred(JNIEnv *env) {
-    return (jthrowable) addJNILref(exceptionOccured());
+    return (jthrowable) addJNILref(exceptionOccurred());
 }
 
 void Jam_ExceptionDescribe(JNIEnv *env) {
@@ -570,7 +570,7 @@ const char *Jam_GetStringUTFChars(JNIEnv *env, jstring string, jboolean *isCopy)
 }
 
 void Jam_ReleaseStringUTFChars(JNIEnv *env, jstring string, const char *chars) {
-    free((void*)chars);
+    sysFree((void*)chars);
 }
 
 jsize Jam_GetArrayLength(JNIEnv *env, jarray array) {
@@ -1421,7 +1421,29 @@ jint parseInitOptions(JavaVMInitArgs *vm_args, InitArgs *args) {
 
         } else if(strcmp(string, "-Xcompactalways") == 0) {
             args->compact_specified = args->do_compact = TRUE;
+#ifdef INLINING
+        } else if(strcmp(string, "-Xnoinlining") == 0) {
+            /* Turning inlining off is equivalent to setting
+               code memory to zero */
+            args->codemem = 0;
 
+        } else if(strncmp(string, "-Xreplication:", 14) == 0) {
+            char *pntr = string + 14;
+
+            if(strcmp(pntr, "none") == 0)
+                args->replication = INT_MAX;
+            else
+                if(strcmp(pntr, "always") == 0)
+                    args->replication = 0;
+                else
+                    args->replication = strtol(pntr, NULL, 0);
+
+        } else if(strncmp(string, "-Xcodemem:", 10) == 0) {
+            char *pntr = string + 10;
+
+            args->codemem = strncmp(pntr, "unlimited", 10) == 0 ?
+                INT_MAX : parseMemValue(pntr);
+#endif
         } else if(!vm_args->ignoreUnrecognized)
             goto error;
     }
@@ -1475,3 +1497,4 @@ void *getJNIInterface() {
     return &Jam_JNINativeInterface;
 }
 #endif
+
