@@ -178,7 +178,7 @@ Object *createConstructorObject(MethodBlock *mb) {
         strcpy(sig, mb->type);
         classes = convertSig2ClassArray(&sig, mb->class);
         exceps = getExceptionTypes(mb);
-        free(signature);
+        sysFree(signature);
 
         if((classes == NULL) || (exceps == NULL))
             return NULL;
@@ -236,7 +236,7 @@ Object *createMethodObject(MethodBlock *mb) {
 
         sig++;
         ret = convertSigElement2Class(&sig, mb->class);
-        free(signature);
+        sysFree(signature);
 
         if((classes == NULL) || (exceps == NULL) || (name == NULL) || (ret == NULL))
             return NULL;
@@ -292,7 +292,7 @@ Object *createFieldObject(FieldBlock *fb) {
 
         strcpy(signature, fb->type);
         type = convertSigElement2Class(&sig, fb->class);
-        free(signature);
+        sysFree(signature);
         name = createString(fb->name);
 
         if((type == NULL) || (name == NULL))
@@ -484,7 +484,7 @@ Class *findClassFromSignature(char *type_name, Class *class) {
     strcpy(name, type_name);
 
     type_class = convertSigElement2Class(&pntr, class);
-    free(name);
+    sysFree(name);
 
     return type_class;
 }
@@ -503,8 +503,8 @@ Object *parseElementValue(Class *class, u1 **data_ptr, int *data_len) {
     switch(tag) {
         default: {
             int cp_tag = CONSTANT_Integer;
+            int prim_type_no = 0;
             int const_val_idx;
-            int prim_type_no;
 
             switch(tag) {
                 case 'Z':
@@ -559,7 +559,7 @@ Object *parseElementValue(Class *class, u1 **data_ptr, int *data_len) {
                 return NULL;
 
             enum_obj = *(Object**)executeStaticMethod(enum_class, enum_valueof_mb, type_class, const_name);
-            if(exceptionOccured())
+            if(exceptionOccurred())
                 return NULL;
 
             return enum_obj;
@@ -606,7 +606,7 @@ Object *parseAnnotation(Class *class, u1 **data_ptr, int *data_len) {
         return NULL;
 
     executeMethod(map, map_init_mb);
-    if(exceptionOccured())
+    if(exceptionOccurred())
         return NULL;
 
     READ_TYPE_INDEX(type_idx, cp, CONSTANT_Utf8, *data_ptr, *data_len);
@@ -627,12 +627,12 @@ Object *parseAnnotation(Class *class, u1 **data_ptr, int *data_len) {
             return NULL;
 
         executeMethod(map, map_put_mb, element_name, element_value);
-        if(exceptionOccured())
+        if(exceptionOccurred())
             return NULL;
     }
 
     anno = *(Object**)executeStaticMethod(anno_inv_class, anno_create_mb, type_class, map);
-    if(exceptionOccured())
+    if(exceptionOccurred())
         return NULL;
 
     return anno;
@@ -875,9 +875,12 @@ Object *invoke(Object *ob, MethodBlock *mb, Object *arg_array, Object *param_typ
 
     Object *excep;
 
-    if(check_access && !checkMethodAccess(mb, getCallerCallerClass())) {
-        signalException("java/lang/IllegalAccessException", "method is not accessible");
-        return NULL;
+    if(check_access) {
+        Class *caller = getCallerCallerClass();
+        if(!checkClassAccess(mb->class, caller) || !checkMethodAccess(mb, caller)) {
+            signalException("java/lang/IllegalAccessException", "method is not accessible");
+            return NULL;
+        }
     }
 
     if(args_len != types_len) {
@@ -909,7 +912,7 @@ Object *invoke(Object *ob, MethodBlock *mb, Object *arg_array, Object *param_typ
 
     POP_TOP_FRAME(ee);
 
-    if((excep = exceptionOccured())) {
+    if((excep = exceptionOccurred())) {
         Object *ite_excep;
         MethodBlock *init;
         Class *ite_class;
@@ -917,7 +920,7 @@ Object *invoke(Object *ob, MethodBlock *mb, Object *arg_array, Object *param_typ
         clearException();        
         ite_class = findSystemClass("java/lang/reflect/InvocationTargetException");
 
-        if(!exceptionOccured() && (ite_excep = allocObject(ite_class)) &&
+        if(!exceptionOccurred() && (ite_excep = allocObject(ite_class)) &&
                         (init = lookupMethod(ite_class, "<init>", "(Ljava/lang/Throwable;)V"))) {
             executeMethod(ite_excep, init, excep);
             setException(ite_excep);
