@@ -169,6 +169,7 @@ void threadYield(Thread *thread) {
 
 void threadInterrupt(Thread *thread) {
     Monitor *mon;
+    Thread *self = threadSelf();
 
     /* monitorWait sets wait_mon _before_ checking interrupted status.  Therefore,
        if wait_mon is null, interrupted status will be noticed.  This guards
@@ -198,7 +199,16 @@ void threadInterrupt(Thread *thread) {
        handler will just return (as in the user doing a kill), and do
        nothing otherwise. */
 
+    /* Note, under Linuxthreads pthread_kill obtains a lock on the thread
+       being signalled.  If another thread is suspending all threads,
+       and the interrupting thread is earlier in the thread list than the
+       thread being interrupted, it can be suspended holding the lock.
+       When the suspending thread tries to signal the interrupted thread
+       it will deadlock.  To prevent this, disable suspension. */
+
+    fastDisableSuspend(self);
     pthread_kill(thread->tid, SIGUSR1);
+    fastEnableSuspend(self);
 }
 
 void *getStackTop(Thread *thread) {
@@ -241,6 +251,7 @@ char *getThreadStateString(Thread *thread) {
         case BLOCKED:
             return "BLOCKED";
     }
+    return "INVALID";
 }
 
 int getThreadsCount() {
@@ -256,11 +267,15 @@ long long getTotalStartedThreadsCount() {
 }
 
 void resetPeakThreadsCount() {
+    Thread *self = threadSelf();
+
     /* Grab the thread lock to protect against
        concurrent update by threads starting/dying */
+    disableSuspend(self);
     pthread_mutex_lock(&lock);
     peak_threads_count = threads_count;
     pthread_mutex_unlock(&lock);
+    enableSuspend(self);
 }
 
 void initialiseJavaStack(ExecEnv *ee) {
@@ -288,7 +303,7 @@ long long javaThreadId(Thread *thread) {
 
 Thread *findHashedThread(Thread *thread, long long id) {
 
-#define FOUND(thread_id)
+#define FOUND(ptr) ptr
 #define DELETED ((void*)-1)
 #define PREPARE(thread_id) thread
 #define SCAVENGE(ptr) ptr == DELETED
@@ -437,7 +452,7 @@ void detachThread(Thread *thread) {
 
     /* If there's an uncaught exception, call uncaughtException on the thread's
        exception handler, or the thread's group if this is unset */
-    if((excep = exceptionOccured())) {
+    if((excep = exceptionOccurred())) {
         FieldBlock *fb = findField(thread_class, "exceptionHandler",
                                                  "Ljava/lang/Thread$UncaughtExceptionHandler;");
         Object *thread_handler = fb == NULL ? NULL : (Object *)INST_DATA(jThread)[fb->offset];
@@ -460,8 +475,11 @@ void detachThread(Thread *thread) {
        point will result in an IllegalThreadStateException */
     INST_DATA(jThread)[vmthread_offset] = 0;
 
+    /* Remove thread from the ID map hash table */
+    deleteThreadFromHash(thread);
+
     /* Disable suspend to protect lock operation */
-    disableSuspend0(thread, &excep);
+    disableSuspend(thread);
 
     /* Grab global lock, and update thread structures protected by
        it (thread list, thread ID and number of daemon threads) */
@@ -495,8 +513,8 @@ void detachThread(Thread *thread) {
        However, they must have a reference to the VMThread -- therefore, it
        is safe to free during GC when the VMThread is determined to be no
        longer reachable. */
-    free(ee->stack);
-    free(ee);
+    sysFree(ee->stack);
+    sysFree(ee);
 
     /* If no more daemon threads notify the main thread (which
        may be waiting to exit VM).  Note, this is not protected
@@ -510,18 +528,16 @@ void detachThread(Thread *thread) {
         pthread_mutex_unlock(&exit_lock);
     }
 
-    /* Remove thread from the ID map hash table */
-    deleteThreadFromHash(thread);
-
     TRACE("Thread 0x%x id: %d detached from VM\n", thread, thread->id);
 }
 
 void *threadStart(void *arg) {
     Thread *thread = (Thread *)arg;
-    ExecEnv *ee = thread->ee;
-    Object *jThread = ee->thread;
-    ClassBlock *cb = CLASS_CB(jThread->class);
-    MethodBlock *run = cb->method_table[run_mtbl_idx];
+    Object *jThread = thread->ee->thread;
+
+    /* Parent thread created thread with suspension disabled.
+       This is inherited so we need to enable */
+    enableSuspend(thread);
 
     /* Complete initialisation of the thread structure, create the thread
        stack and add the thread to the thread list */
@@ -531,7 +547,7 @@ void *threadStart(void *arg) {
     addThreadToHash(thread);
 
     /* Execute the thread's run method */
-    executeMethod(jThread, run);
+    executeMethod(jThread, CLASS_CB(jThread->class)->method_table[run_mtbl_idx]);
 
     /* Run has completed.  Detach the thread from the VM and exit */
     detachThread(thread);
@@ -576,8 +592,8 @@ void createJavaThread(Object *jThread, long long stack_size) {
 
     if(pthread_create(&thread->tid, &attributes, threadStart, thread)) {
         INST_DATA(jThread)[vmthread_offset] = 0;
-        free(ee);
-//        free(thread);
+        sysFree(ee);
+        enableSuspend(self);
         signalException("java/lang/OutOfMemoryError", "can't create thread");
         return;
     }
@@ -630,7 +646,7 @@ static void *shell(void *args) {
     attachThread(((char**)args)[0], TRUE, &self, self,
                  (Object*)INST_DATA(main_ee.thread)[group_offset]);
 
-    free(args);
+    sysFree(args);
     (*(void(*)(Thread*))start)(self);
     return NULL;
 }
@@ -842,6 +858,7 @@ void fastEnableSuspend(Thread *thread) {
 }
 
 void dumpThreadsLoop(Thread *self) {
+    char buffer[256];
     Thread *thread;
     sigset_t mask;
     int sig;
@@ -865,24 +882,27 @@ void dumpThreadsLoop(Thread *self) {
 
         for(thread = &main_thread; thread != NULL; thread = thread->next) {
             uintptr_t *thr_data = INST_DATA(thread->ee->thread);
-            char *name = String2Cstr((Object*)thr_data[name_offset]);
             int priority = thr_data[priority_offset];
             int daemon = thr_data[daemon_offset];
             Frame *last = thread->ee->last_frame;
 
+            /* Get thread name; we don't use String2Cstr(), as this mallocs memory
+               and may deadlock with a thread suspended in malloc/realloc/free */
+            String2Buff((Object*)thr_data[name_offset], buffer, sizeof(buffer));
+
             jam_printf("\n\"%s\"%s %p priority: %d tid: %p id: %d state: %s (%d)\n",
-                  name, daemon ? " (daemon)" : "", thread, priority, thread->tid,
+                  buffer, daemon ? " (daemon)" : "", thread, priority, thread->tid,
                   thread->id, getThreadStateString(thread), thread->state);
-            free(name);
 
             while(last->prev != NULL) {
                 for(; last->mb != NULL; last = last->prev) {
                     MethodBlock *mb = last->mb;
                     ClassBlock *cb = CLASS_CB(mb->class);
-                    char *dot_name = slash2dots(cb->name);
 
-                    jam_printf("\tat %s.%s(", dot_name, mb->name);
-                    free(dot_name);
+                    /* Convert slashes in class name to dots.  Similar to above,
+                       we don't use slash2dots(), as this mallocs memory */
+                    slash2dots2buff(cb->name, buffer, sizeof(buffer)); 
+                    jam_printf("\tat %s.%s(", buffer, mb->name);
 
                     if(mb->access_flags & ACC_NATIVE)
                         jam_printf("Native method");
@@ -1106,6 +1126,7 @@ void initialiseMainThread(InitArgs *args) {
     return;
 
 error:
-    jam_fprintf(stderr, "Error initialising VM (initialiseMainThread)\n");
+    jam_fprintf(stderr, "Error initialising VM (initialiseMainThread)\nCheck "
+                        "the README for compatible versions of GNU Classpath\n");
     exitVM(1);
 }
